<?php

class PmConnector {
	
	var $conn; // mysql connection resource (as returned by mysql_connect)
	var $languages; // array holding an ordered list of content languages to be considered for the current connection
	var $userId; // id of the PTM user
	var $userGroups; // array holding the ids of groups the user belongs to
	var $pageIdImages; // internal id of the images root page
	var $metadata; // array holding a conversion table for metadata tags
	var $globals; // array holding Pixtacy's global variables
	var $request; // array holding all the parameters of the original request
	
	var $mysqlServer;
	var $mysqlDatabase;
	var $mysqlUser;
	var $mysqlPassword;
	var $mysqlTablePrefix;
	
	var $cfg; // array holding the configuration parameters of the Pixtacy environment

	var $tblContent;
	var $tblImages;
	var $tblImagelists;
	var $tblUsers;
	var $tblIndex;
	var $tblRoot;
	
	function PmConnector(){
		$path = realpath( dirname( __FILE__ ) . '/../../../config/config.php' );
		if( $path && $cfg = parse_ini_file( $path ) ){

			$this->cfg = $cfg;
			
			$this->mysqlServer = $cfg['mysqlServer'];
			$this->mysqlDatabase = $cfg['mysqlDatabase'];
			$this->mysqlUser = $cfg['mysqlUser'];
			$this->mysqlPassword = $cfg['mysqlPassword'];
			$this->mysqlTablePrefix = $cfg['mysqlTablePrefix'];

			$this->tblContent = $this->mysqlTablePrefix . 'content';
			$this->tblImages = $this->mysqlTablePrefix . 'data_image';
			$this->tblImagelists = $this->mysqlTablePrefix . 'data_imagelist';
			$this->tblUsers = $this->mysqlTablePrefix . 'user';
			$this->tblIndex = $this->mysqlTablePrefix . 'index';
			$this->tblRoot = $this->mysqlTablePrefix . 'data_vtroot';
			
			$this->metadata['keywords'] = 'keywords';
			$this->metadata['date_created'] = 'creationdate';
			$this->metadata['by_line'] = 'photographer';
			$this->metadata['headline'] = 'name';
			$this->metadata['credit'] = 'credit';
			$this->metadata['copyright_notice'] = 'copyright';
			$this->metadata['caption'] = 'remarks';
			$this->metadata['author_code'] = 'photographer';
			$this->metadata['city'] = 'city';
			$this->metadata['country_primary_location_name'] = 'country';
			$this->metadata['country_primary_location_code'] = 'countrycode';
			$this->metadata['province_state'] = 'state';
			$this->metadata['special_instructions'] = 'instructions';
		}
	}
	
	function connect( $request ){
		if( isset( $this->conn ) && $this->conn !== false ){
			return true;
		}
		if( ! $this->mysqlServer ){
			$this->error('configuration data missing');
			return false;
		}
		if( ! $this->conn = mysql_connect( $this->mysqlServer, $this->mysqlUser, $this->mysqlPassword ) ) {
			$this->error('could not connect to mysql server');
			return false;
		}
		if( ! mysql_select_db( $this->mysqlDatabase, $this->conn ) ){
			$this->error('mysql database does not exist');
			return false;
		}
		if( ! $this->userId = $this->get_user_id( $request['username'], $request['password'] ) ){
			$this->error('login failed');
			return false;
		}
		if( ! $this->pageIdImages = $this->get_images_page_id() ){
			$this->error('could not find root images page');
			return false;
		}
		$this->request = $request;
		return true;
	}
	
	function disconnect(){
		if( isset( $this->conn ) && $this->conn !== false ){
			mysql_close( $this->conn );
		}
	}

	function count( $search = array() ){
		$conditions = $this->build_where_clause( $search );
		$query = "SELECT COUNT(*) AS count FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i WHERE $conditions";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return -1;
		}
		if( ! $row = mysql_fetch_assoc( $result ) ){
			return -1;
		}
		mysql_free_result( $result );
		return $row['count'];
	}
	
	function search( $search, $maxresults = 0, $sortfield = 'datearchivation', $sortorder = 'desc'){
		// $search = array( $criteria1, $criteria2, ... )
		// $criteria = $comparison or $or_block
		// $or_block = array( $comparison, $comparison, ... )
		// $comparison = array( $field, $operator, $value )
		// $operator = contains|equal|greater|greater-or-equal|less|less-or-equal|
		//             not-contains|not-equal|not-greater etc.

		$conditions = $this->build_where_clause( $search );
		$sorting = $this->build_order_clause( $sortfield, $sortorder );

		$query = "SELECT LEFT( d.`dateiname`, LENGTH( d.`dateiname` ) - 4 ) as id, "
					."DATE_FORMAT( FROM_UNIXTIME( c.`creationTime` ), '%Y%m%d' ) AS archiveDate, "
					."d.`creationdate` AS exposureDate "
					."FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i "
					."WHERE $conditions ORDER BY $sorting";
		if( $maxresults > 0 ){
			$query .= " LIMIT 0,$maxresults";
		}
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return -1;
		}

		$images = array();
		while( $row = mysql_fetch_assoc( $result ) ){
			$images[] = $row;
		}

		if( mysql_num_rows( $result ) == 0 ){
			$count = 0;
		} elseif( $maxresults == 0 || mysql_num_rows( $result ) < $maxresults ){
			$count = -1;
		} else {
			// get total number of found images
			mysql_free_result( $result );
			$query = "SELECT COUNT(*) AS count FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i WHERE $conditions";
			if( ! $result = mysql_query( $query, $this->conn ) ){
				return -1;
			}
			if( ! $row = mysql_fetch_assoc( $result ) ){
				return -1;
			}
			$count = $row['count'];
			mysql_free_result( $result );
			if( $count == $maxresults ){
				$count = -1;
			}
		}

		return array( 'count' => $count, 'assets' => $images );
	}
	
	function asset_metadata( $ref, $tags = array() ){
		// returns metadata for a specific asset
		// return array( array( 'field' => 'keywords', 'value' => 'woman' ), array( 'field' => 'date_created', 'value' => '20100923' ) )
		if( empty( $tags ) ){
			$tags = $this->metadata_list();
		}
		$columns = $this->tag2column( $tags );
		$fieldlist = '`' . implode( '`,`', $columns ) . '`';
		$query = "SELECT $fieldlist FROM `{$this->tblImages}` WHERE `dateiname` LIKE '$ref%'";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return -1;
		}
		if( ! $row = mysql_fetch_assoc( $result ) ){
			return -2;
		}
		mysql_free_result( $result );
		$result = array();
		foreach( $columns as $column ){
			if( $column == 'keywords'){
				foreach( explode( ',', $row['keywords'] ) as $keyword ){
					$result[] = array( 'field' => 'keywords', 'value' => html_entity_decode( trim( $keyword ) ) );
				}
			} else {
				$result[] = array( 'field' => $this->column2tag( $column ), 'value' => strip_tags( html_entity_decode( $row[ $column ] ) ) );
			}
		}
		return $result;
	}
	
	function asset_permissions( $ref, $clearing = false ){
		// returns permissions and clearing information for a specific asset

		foreach( array( 'view', 'download' ) as $type ){
			foreach( array( 'thumb', 'preview', 'highres' ) as $resolution ){
				if( $this->get_permission( $ref, $resolution, $type ) ){
					$result[ $type ][ $resolution ] = '1';
				} else {
					$result[ $type ][ $resolution ] = '0';
				}
			}
		}
		
		if( $result['download']['highres'] == '1' ){
			$hiresFileIsDownloadble = false;
			$query = "SELECT `verzeichnis`, `dateiname` FROM `{$this->tblImages}` WHERE `dateiname` LIKE '$ref%'";
			if( $query = mysql_query( $query, $this->conn ) ){
				if( $row = mysql_fetch_assoc( $query ) ){
					$this->load_globals();
					$dir = $this->globals['pxtcPathHires'];
					if( $dir[0] != '/' ){
						$dir = dirname( __FILE__ ) . '/../../../' . $dir;
					}
					$file = "$dir/{$row['verzeichnis']}/{$row['dateiname']}";
					if( $file = realpath( $file ) ){
						$hiresFileIsDownloadble = true;
					}
				}
			}
			$result['download']['highres'] = $hiresFileIsDownloadble ? '1' : '0';
		}
		return $result;
	}
	
	function download( $ref, $resolution, $download = true, $comment = '' ){
		// returns a file reference of a specific asset
		
		$query = "SELECT `verzeichnis`, `dateiname` FROM `{$this->tblImages}` WHERE `dateiname` LIKE '$ref%'";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return -1;
		}
		if( ! $row = mysql_fetch_assoc( $result ) ){
			return -2;
		}

		mysql_free_result( $result );

		$resArray = array( '1' => 'thumb', '2' => 'preview', '3' => 'highres' );
		if( !in_array( $resolution, $resArray ) ){
			if( !isset( $resArray[ $resolution ] ) ){
				return -3;
			}
			$resolution = $resArray[ $resolution ];
		}
		
		if( $download ){
			$type = 'download';
		} else {
			$type = 'view';
		}
		
		if( !$this->get_permission( $ref, $resolution, $type ) ){
			return -4;
		}

		if( !$this->load_globals() ){
			return -1;
		}

		switch( $resolution ){
			case 'thumb':
				$dir = "data/pixtacy_thumbnails";
				$notify = 0;
				break;
			case 'preview':
				$dir = $this->globals['pxtcPathLores'];
				$notify = $this->globals['pxtcNotifyLoresDownload'];
				break;
			case 'highres':
				$dir = $this->globals['pxtcPathHires'];
				$notify = $this->globals['pxtcNotifyHiresDownload'];
				break;
		}
		if( $dir[0] != '/' ){
			$dir = dirname( __FILE__ ) . '/../../../' . $dir;
		}
		$file = "$dir/{$row['verzeichnis']}/{$row['dateiname']}";
		if( ! $file = realpath( $file ) ){
			return -5;
		}
		
		if( !empty( $this->globals['mailRecipient'] ) && $notify && $download ){
			$subject = 'Pixtacy-Download ber Picturemaxx';
			$message = 'ber Picturemaxx wurde am ' . date('d.m.Y') . ' um ' . date('H:i') 
						. " Uhr die folgende Bilddatei heruntergeladen:\n"
						. "\n"
						. "$file\n"
						. "\n"
						. "Kundennummer: {$this->request['client_id']}\n"
						. "Kundengruppe(n): {$this->request['client_group_name']}\n"
						. "Land/Sprache: {$this->request['client_country_name_iso3']}/{$this->request['client_language_name_iso3']}\n"
						;
			if( $comment ){
				$message .= "\nBemerkungen:\n$comment\n";
			}
			mail( $this->globals['mailRecipient'], $subject, $message );
		}

		return $file;
	}
	
	function download_app( $assets = array() ){
		// returns url to a web page where user can increase credit limit
	}
	
	function author_list(){
		// returns a list of authors (photographers or agencies)
		$conditions = $this->build_where_clause();
		$query = "SELECT d.`photographer` FROM `{$this->tblContent}` c, `{$this->tblImages}` d, `{$this->tblIndex}` i WHERE $conditions GROUP BY d.`photographer`";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return -1;
		}
		$authorlist = array();
		while( $row = mysql_fetch_assoc( $result ) ){
			if( !empty( $row['photographer'] ) ){
				$authorlist[] = array( 'ref' => $row['photographer'], 'name' => $row['photographer'] );
			}
		}
		mysql_free_result( $result );
		return $authorlist;
	}
	
	function offer_list( $datefrom, $dateto, $maxresults = 0 ){
		// returns a list of offers
		return array();
	}
	
	function offer_assets( $ref, $maxresults = 0 ){
		// returns a list of assets for a specific offer
		return array();
	}
	
	function metadata_list(){
		// returns a list of supported metadata fields
		return array_keys( $this->metadata );
	}
	
	#########################
	#### Private Methods ####
	#########################
	
	function error( $msg ){
		if( isset( $this->cfg['log'] ) && $this->cfg['log'] == '1' ){
			echo $msg;
		}
	}

	function get_user_id( $username, $password ){
		// checks if $username and $password are correct and returns id of the user
		$hash = md5( $password );
		$query = "SELECT `id` FROM `{$this->tblUsers}` WHERE `name` = '$username' AND `active`= 1 AND `password` = '$hash'";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return false;
		}
		if( ! $row = mysql_fetch_assoc( $result ) ){
			return false;
		}
		mysql_free_result( $result );
		return $row['id'];
	}


	function get_images_page_id(){
		$query = "SELECT c2.`id` AS id FROM `{$this->tblContent}` AS c1 INNER JOIN `{$this->tblContent}` AS c2 ON c1.`id` = c2.`containerID` WHERE c1.`package` = 'pixtacy' AND c1.`type` = 'index' AND c2.`package` = 'pixtacy' AND c2.`type` = 'imagelist'";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return false;
		}
		if( ! $row = mysql_fetch_assoc( $result ) ){
			return false;
		}
		mysql_free_result( $result );
		return $row['id'];
	}


	function get_user_groups( $type = 'id' ){
		if( !empty( $this->userGroups ) ){
			return $this->userGroups[$type];
		}
		if( $type != 'name' ){
			$type = 'id';
		}
		$query = "SELECT c1.`id`, c1.`name` FROM `{$this->tblContent}` AS c1 INNER JOIN `{$this->tblContent}` AS c2 ON c1.`id` = c2.`containerID` WHERE c2.`aliasID` = {$this->userId}";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return false;
		}
		$groupIds = array();
		$groupNames = array();
		while( $row = mysql_fetch_assoc( $result ) ){
			$groupIds[] = $row['id'];
			$groupNames[] = $row['name'];
		}
		mysql_free_result( $result );
		$this->userGroups['id'] = $groupIds;
		$this->userGroups['name'] = $groupNames;
		return $this->userGroups[$type];
	}


	function build_where_clause( $search = array() ){
		$userIds = implode( ',', array_merge( array( $this->userId ), $this->get_user_groups() ) );
		$conditions = " c.`id` = d.`vtid` AND c.`id` = i.`id` AND c.`package` = 'pixtacy' AND c.`type` = 'image' "
						."AND FIND_IN_SET( {$this->pageIdImages}, c.`path` ) "
						."AND ( ( c.`ownerID` IN ($userIds) OR ( c.`userID` IN ($userIds) "
						."AND LEFT( RIGHT( c.`accessRights`, 2 ), 1 ) > 3 ) ) "
						."OR RIGHT( c.accessRights, 1 ) > 3 ) AND c.`userID` != 52";

		foreach( $search as $criteria ){
			if( isset( $criteria['field'] ) ){
				// single criteria
				$str = $this->build_condition( $criteria );
				if( $str !== false ){
					$conditions .= ' AND ' . $str;
				}
			} else {
				// OR Block
				$conditions .= ' AND ( ';
				$counter = 1;
				foreach( $criteria as $subcriteria ){
					$str = $this->build_condition( $subcriteria );
					if( $str !== false ){
						$conditions .= ( $counter > 1 ? ' OR ' : '' ) . $str;
						$counter++;
					}
				}
				$conditions .= ' ) ';
			}
		}
		return $conditions;
	}
	
	function build_order_clause( $sortfield, $sortorder ){
		switch( $sortfield ){
			case 'datearchivation':
				$str = "c.`creationTime`";
				break;
			case 'reference':
				$str = "d.`dateiname`";
				break;
			default:
				$str = "d.creationDate";
		}
		if( $sortorder == 'asc' ){
			$str .= ' ASC';
		} else {
			$str .= ' DESC';
		}
		return $str;
	}

	function build_condition( $criteria ){
		// erzeugt einen String, der als Bedingung in einer MySQL-Where-Klausel benutzt werden kann
		// $criteria muss ein assoziatives Array mit den Elementen 'field', 'operator', 'value' und optional 'metadataTag' sein
		extract( $criteria );
		$value = utf8_decode( $value );
		$str = '';
		switch( $operator ){
			case 'not-contains':
				$str = ' NOT';
			case 'contains':
				$value = addcslashes( strtolower($value), '.\/+[](){}' );
				$value = str_replace( '?', '.', $value );
				$value = str_replace( '*', '.+', $value );
				$value = str_replace( array( 'ae', '' ), '(ae|)', $value );
				$value = str_replace( array( 'oe', '' ), '(oe|)', $value );
				$value = str_replace( array( 'ue', '' ), '(ue|)', $value );
				$value = str_replace( array( 'ss', '' ), '(ss|)', $value );
				$str .= " REGEXP '[[:<:]]{$value}[[:>:]]' ";
				break;
			case 'equal':
				$str = " = '$value' ";
				break;
			case 'not-equal':
				$str = " != '$value' ";
				break;
			case 'greater':
			case 'not-less-or-equal':
				$str = " > '$value' ";
				break;
			case 'greater-or-equal':
			case 'not-less':
				$str = " >= $value ";
				break;
			case 'less':
			case 'not-greater-or-equal':
				$str = " < $value ";
				break;
			case 'less-or-equal':
			case 'not-greater':
				$str = " <= $value ";
				break;
		}

		if( empty( $str ) ){
			return false;
		}

		switch( $field ){
			case 'author':
				$result = 'd.`photographer` ' . $str;
				break;
			case 'datecreated':
				$result = 'd.`creationdate` ' . $str;
				break;
			case 'optical':
				$result = '1'; // Pixtacy-Bilder liegen immer in digitaler Form vor
				break;
			case 'fulltext':
				$result = 'i.`index` ' . $str;
				break;
			case 'metadata':
				if( $column = $this->tag2column( $metadataTag ) ){
					$result = "d.`$column` $str";
				}
				break;
			case 'mimegroup':
				if( $value == 'image' ){
					$result = '1';
				} else {
					$result = '1 = 0';
					// TODO: Bilder und Videos differenzieren
				}
				break;
			case 'format':
				$result = 'd.`format` ' . $str;
				break;
			case 'collection':
			case 'collectiontype':
				$result = '1';
				// TODO: Properties 'collection' und 'collectiontype' untersttzen
				break;
		}

		if( empty( $result ) ){
			return false;
		}

		return ' ' . $result . ' ';
	}
	
	function tag2column( $tag ){
		// returns internal db column name for Picturemaxx tag name
		// $tag can be a single tag or a list of tags
		if( empty( $tag ) ){
			return false;
		}
		if( !is_array( $tag ) && isset( $this->metadata[ $tag ] ) ){
			return $this->metadata[ $tag ];
		}
		foreach( $tag as $item ){
			if( isset( $this->metadata[ $item ] ) ){
				$result[] = $this->metadata[ $item ];
			}
		}
		return $result;
	}

	function column2tag( $column ){
		// returns Picturemaxx tag name for internal db column name
		$trans = array_flip( $this->metadata );
		if( isset( $trans[ $column ] ) ){
			return $trans[ $column ];
		} else {
			return false;
		}
	}
	
	function load_globals(){
		if( !empty( $this->globals ) ){
			return true;
		}
		$query = "SELECT `data` FROM {$this->tblRoot} WHERE `vtid` = 1";
		if( ! $result = mysql_query( $query, $this->conn ) ){
			return false;
		}
		if( ! $row = mysql_fetch_assoc( $result ) ){
			return false;
		}
		$data = unserialize( $row['data'] );
		$this->globals = $data['globals'];
		return true;
	}

	function get_permission( $ref, $resolution, $type = 'download' ){
		
		if( !$this->load_globals() ){
			return false;
		}

		if( !in_array( $type, array( 'view', 'download' ) ) ){
			return false;
		}
		
		$permission = false;

		switch( $resolution ){

			case 'thumb':
				$permission = true;
				break;

			case 'preview':
				if( $type == 'view' ){
					$permission = true;
				} else {
					switch( $this->globals['pxtcPrivilegeLoresDownload'] ){
						case '1':
						case '2':
							$permission = true;
							break;
						case '3':
							$loresGroups = explode( ',', $this->globals['pxtcGroupsLoresDownload'] );
							if( array_intersect( $loresGroups, $this->get_user_groups('name') ) ){
								$permission = true;
							}
					}
				}
				break;

			case 'highres':
				if( $type == 'download' ){
					switch( $this->globals['pxtcPrivilegeHiresDownload'] ){
						case '1':
						case '2':
							$permission = true;
							break;
						case '3':
							$hiresGroups = explode( ',', $this->globals['pxtcGroupsHiresDownload'] );
							if( array_intersect( $hiresGroups, $this->get_user_groups('name') ) ){
								$permission = true;
							}
					}

					if( !$permission ){
						$query = "SELECT c.`accessRights`, d.`downloadallowed` FROM `{$this->tblContent}` c, `{$this->tblImagelists}` d WHERE c.`containerID` = d.`vtid` AND d.`dateiname` LIKE '$ref%'";
						if( ! $result = mysql_query( $query, $this->conn ) ){
							return -1;
						}
						if( ! $row = mysql_fetch_assoc( $result ) ){
							return -2;
						}
						if( substr( $row['accessRights'], -1 ) < 4 && $row['downloadallowed'] == '1' ){
							$permission = true;
						}
						mysql_free_result( $result );
					}
				}
		}
		
		return $permission;
	}
}

?>
