<?php

	function makeThumbnail( $image, $outputFile, $force=false ){
		
		$v = new virthos;
		$size = $v->vtconfig( 'pxtcThumbnailSize' );
		if( (int) $size > 0 ){
			$max_width = $size;
			$max_height = $size;
		} else {
			$max_width = 120;
			$max_height = 120;
		}
		
		if( !file_exists( $image ) ){
			return -1;
		}
		
		if( file_exists( $outputFile ) && filemtime( $outputFile ) >= filemtime( $image ) && !$force ){
			return 0;
		}
		
		$parts = explode( '/', dirname( $outputFile ) );
		$dir = '';
		foreach( $parts as $part ) {
			$dir .= $part . '/';
			if( !is_dir( $dir ) && strlen( $dir ) > 0 ){
				mkdir( $dir );
				chmod( $dir, 0777 );
			}
		}

		$groesse = getimagesize($image); 
		$width = $groesse[0];
		$height = $groesse[1];

		$image_ratio = $width / $height;
		$destination_ratio = $max_width / $max_height;

		$new_width = $max_width;
		$new_height = $max_height;

		if ($width > $max_width AND $height < $width){
	  		$new_height = ceil($max_width / $image_ratio);
		} elseif($height > $max_height AND $height > $width){
	  		$new_width = ceil($max_height * $image_ratio);
		} elseif ($height < $max_height AND $height > $width){
	  		$new_width = ceil($max_height * $image_ratio);
		} elseif ($width < $max_width AND $height < $width){
	  		$new_height = ceil($max_width * $image_ratio);
		} elseif ($height == $width){
			$new_height = $new_width;
		}

		$image_p = imagecreatetruecolor( $new_width, $new_height);
		$image = imagecreatefromjpeg( $image );
		imagecopyresampled( $image_p, $image, 0, 0, 0, 0, $new_width, $new_height, $width, $height );
		imagejpeg( $image_p, $outputFile );
		chmod( $outputFile, 0777);
		return 1;
	}
	
	function getIPTCHeader( $image, $header, $charset = '' ){

		$fieldNames = array( 
			'Category' => '015',
			'SupCategory' => '020',
			'Keywords' => '025',
			'ReferenceNumber' => '050',
			'CreationDate' => '055',
			'CreationTime' => '060',
			'Photographer' => '080',
			'ByLine' => '080',
			'ByLineTitle' => '085',
			'City' => '090',
			'State' => '095',
			'CountryCode' => '100',
			'Country' => '101',
			'Credit' => '110',
			'Title' => '105',
			'Copyright' => '116',
			'Description' => '120',
			'Instructions' => '040',
			'Writer' => '122'
		);
		
		if( !isset( $fieldNames[ $header ] ) ){
			return '';
		}

		if( isset( $GLOBALS['pxtc']['iptc_data'][ $image ] ) ){
			$iptc = $GLOBALS['pxtc']['iptc_data'][ $image ];
		} else {
			$GLOBALS['pxtc']['iptc_data'][ $image ] = array();
			if( !getimagesize( $image, $info ) ){
				return '';
			}
			if( !isset( $info['APP13'] ) ){
				return '';
			}
			$iptc = iptcparse( $info['APP13'] );
		}

		if( !$iptc ){
			return '';
		}

		$GLOBALS['pxtc']['iptc_data'][ $image ] = $iptc;
		$code = '2#' . $fieldNames[ $header ];

		if( !isset( $iptc[ $code ] ) ){
			return '';
		}

		if( is_array( $iptc[ $code ] ) ){
			$result = implode( ', ', $iptc[ $code ] );
		} else {
			$result = $iptc[ $code ];			
		}
		
		if( $charset ){
			require_once( dirname(__FILE__) . '/ConvertCharset.class.php' );
			$Coder = new ConvertCharset( $charset, 'iso-8859-1', false );
			return $Coder->Convert( $result );
		} else {
			return $result;
		}
	}
	
	function getDirectoryList( $path, $as_array = FALSE ){
		if( substr( $path, -1 ) != '/' ){
			$path .= '/';
		}
		$list = array();
		$handle = opendir( $path );

		if( $handle ){
			while( $file = readdir( $handle ) ){
				if( $file != '.' && $file != '..' && substr($file,0,1) != '.' ){
					if( is_dir( $path.$file ) ) {
						$list[] = $file; 
					}
		 		} 
			}
			closedir( $handle );
			sort( $list );
			if( $as_array ){
				return $list;
			} else {
				return implode( ',', $list );
			}
		} else {
	    	return -1;
		}
	}

	function getDirectoryTree( $path, $tree = '', $basePath = '' ){
		$handle = opendir( $path );
		if( $handle ){
			if( $basePath == '' ){
				$basePath = $path;
			}
			if( $tree == '' ){
				$tree = '/';
			} else {
				$tree .= ', ' . substr( $path, strlen( $basePath ) );
			}
			while( false !== ( $item = readdir( $handle ) ) ){
				//echo $path.'/'.$item;
				if( $item != '.' && $item != '..' && substr($item,0,1) != '.' && is_dir( $path.'/'.$item ) ){
					$tree = getDirectoryTree( $path.'/'.$item, $tree, $basePath ); 
		 		} 
			}
			closedir( $handle );
			return $tree;
		} else {
	    	return false;
		}
	}

	function getFileList( $path, $pattern='.*', $as_array = FALSE ){
		if( substr( $path, -1 ) != '/' ){
			$path .= '/';
		}
		$list = array();
		$handle = opendir($path);

		if( $handle ){
			while( false !== ($file = readdir($handle)) ){
				if( $file != '.' && $file != '..' && substr($file,0,1) != '.' ){
					if( is_file( $path.$file ) && preg_match( '/' . $pattern . '/', $file ) > 0 ) {
						$list[] = $file; 
					}
		 		} 
			}
			closedir($handle);
			if( $as_array ){
				return $list;
			} else {
				return implode( ',', $list );
			}
		} else {
	    	return -1;
		}
		
	}
	
	function getFileSize( $file ) { 
		$units = array( 'B', 'kB', 'MB', 'GB', 'TB' ); 
		$bytes = max( filesize( $file ), 0 ); 
		$pow = floor( ($bytes ? log($bytes) : 0) / log(1024) ); 
		$pow = min($pow, count($units) - 1); 
		$bytes /= pow(1024, $pow); 
		return round($bytes, $precision) . ' ' . $units[$pow]; 
	} 

	function getFileInfo( $file, $info ) { 
		$stats = stat( $file ); 
		return $stats[$info]; 
	} 

	function imageName2URL( $str, $tolower = TRUE ){
		// Ascii-Zeichen > 127 mssen in $str als HTML-Entites codiert sein!
		$str = trim($str);
		if( $tolower ){
			$str = strtolower( $str );
		}
		$search = array( '&Auml;','&Ouml;','&Uuml;','&auml;','&ouml;','&uuml;','&szlig;','&eacute;','&egrave;','&ecirc;','&aacute;','&agrave;','&acirc;' );
		$replace = array( 'Ae','Oe','Ue','ae','oe','ue','ss','e','e','e','a','a','a' );
		$str = str_replace( $search, $replace, $str );
		$str = preg_replace( '/[^a-zA-Z0-9-_@\.\s]/', '-', $str );
		$str = preg_replace( '/-+/', '-', $str );
		return $str;
	}

	function encodeNumberList( $list ){
		$arr = array_unique( explode( ',', $list ) );
		$result = array();
		foreach( $arr as $item ){
			if( $item > 0 ){
				$result[] = base_convert( $item, 10, 35 );
			}
		}
		return implode( 'z', $result );
	}

	function decodeNumberList( $list ){
		$arr = explode( 'z', $list );
		$result = array();
		foreach( $arr as $item ){
			$result[] = base_convert( $item, 35, 10 );
		}
		return implode( ',', $result );
	}

	function str_convert_JS2PHP( $str ){
		require_once( 'templates/pixtacy/resources/ConvertCharset.class.php' );
		$callback = create_function( '$matches', '$cc = new ConvertCharset( \'utf-8\', \'iso-8859-1\' ); return $cc->Convert( chr( hexdec($matches[0]) ) );' );
		//$callback = create_function( '$matches', 'return $matches[0];' );
		return preg_replace_callback( '/\\\\u([0-9a-f]{4})/', $callback, $str );
	}

	function addToList( $jList, $jObject ){
		if( $jList ){
			$list = json_decode( utf8_encode($jList), TRUE );
			if( $jObject ){
				$list[] = json_decode( utf8_encode($jObject), TRUE );
			}
			return str_convert_JS2PHP( json_encode($list) );
		} else {
			if( $jObject ){
				return '[' . $jObject . ']';
			} else {
				return '';
			}
		}
	}

	function setAttribute( $jObject, $attribName, $attribValue ){
		if( $jObject ){
			if( substr( trim($jObject), 0, 1 ) != '[' ){
				$jObject = '[' . $jObject . ']';
			}
			$arr = json_decode( utf8_encode($jObject), TRUE );
			$object = (array)$arr[0];
		} else {
			$object = array();
		}
		if( $attribName ){
			$object[utf8_encode($attribName)] = utf8_encode($attribValue);
		}
		return str_convert_JS2PHP( json_encode($object) );
	}

	function fetchObject( $jList, $keyName, $keyValue ){
		// Finds first object in $jList where $keyName attribute is $keyValue
		// and returns it in JSON format
		if( $jList && $keyName && $keyValue ){
			$list = json_decode( utf8_encode($jList), TRUE );
			foreach( $list as $object ){
				if( $object[ $keyName ] == $keyValue ){
					return str_convert_JS2PHP( json_encode( $object ) );
				}
			}
		}

		return '';
	}
	
	function getAttribute( $jObject, $attribName ){
		if( $jObject && $attribName ){
			$obj = json_decode( utf8_encode($jObject), TRUE );
			return $obj[ $attribName ];
		} else {
			return '';
		}
	}

	function countList( $jList ){
		if( $jList ){
			$list = json_decode( utf8_encode($jList), TRUE );
			return count( $list );
		} else {
			return 0;
		}
	}

	function evalList( $jList, $attribute, $operator='sum' ){
		if( $jList ){
			$list = json_decode( utf8_encode($jList), TRUE );
			$result = 0;
			foreach( $list as $item ){
				switch( $operator ){
					case 'max' :
						$result = max( $item[ $attribute ], $result );
						break;
					case 'sum' :
						$result += str_replace( ',', '.', $item[ $attribute ]);
						break;
				}
			}
			return $result;
		} else {
			return false;
		}
	}

	function removeFromList( $jList, $position ){
		if( !$jList ){
			return '';
		}
		$list = json_decode( utf8_encode($jList), TRUE );
		if( $position < 1 || $position > count($list) ){
			return $jList;
		}
		array_splice( $list, $position-1, 1 );
		if( count($list) ){
			return str_convert_JS2PHP( json_encode( $list ) );
		} else {
			return '';
		}
	}
	
	function getListItem( $jList, $position ){
		if( !$jList ){
			return '';
		}
		$list = json_decode( utf8_encode($jList), TRUE );
		if( $position < 1 || $position > count($list) ){
			return '';
		}
		return str_convert_JS2PHP( json_encode( $list[$position-1] ) );
	}
	
	function replaceListItem( $jList, $position, $jObject ){
		if( !$jList ){
			return '';
		}
		$list = json_decode( utf8_encode($jList), TRUE );
		if( $position < 1 || $position > count($list) || !$jObject ){
			return $jList;
		}
		$list[$position-1] = json_decode( utf8_encode($jObject), TRUE );
		return str_convert_JS2PHP( json_encode( $list ) );
	}
	
	function createDownload( $path, $reference, $expirationDays = 1 ){
		if( substr( $path, 0, 1 ) == '/' && !file_exists( $path ) ){
			return '';
		}
		$v = new virthos;
		$path = mysql_real_escape_string( $path );
		$table = $v->vtconfig('mysqlTablePrefix') . 'table_downloads';
		$query = "CREATE TABLE IF NOT EXISTS `$table` (
			`code` varchar(32) NOT NULL default '',
			`path` text default '',
			`reference` varchar(64) NOT NULL default '',
			`created` datetime NOT NULL default '0000-00-00 00:00:00',
			`expires` datetime NOT NULL default '0000-00-00 00:00:00',
			`download_ts` datetime NOT NULL default '0000-00-00 00:00:00',
			`download_ip` varchar(15) NOT NULL default '',
			PRIMARY KEY  (`code`)
			)";
		mysql_query( $query );
		do {
			$code = md5( rand( 1000, 9999 ) . microtime() );
			$query = "INSERT INTO $table (`code`,`path`,`reference`,`created`,`expires`) VALUES ('$code','$path','$reference', NOW(), DATE_ADD( CURRENT_TIMESTAMP, INTERVAL $expirationDays DAY ) )";
			$result = mysql_query( $query );
		} while( $result && !mysql_affected_rows() );
		
		return $code;
	}

	function deleteDownloads( $reference ){
		$v = new virthos;
		$table = $v->vtconfig('mysqlTablePrefix') . 'table_downloads';
		if( $reference ){
			$query = "DELETE FROM $table WHERE reference = '$reference'";
			mysql_query( $query );
		}
	}

	function getImageOrientation( $file, $precision = 0 ){
		if( ! file_exists( $file ) ){
			return '';
		}
		list( $width, $height, $type, $attr ) = getimagesize( $file );
		if( $precision > 0 && abs( $height - $width ) / min( $height, $width ) < $precision ){
			return 'square';
		} else {
			return $height > $width ? 'portrait' : 'landscape';
		}
	}
	
	function getImageLongSide( $file ){
		if( ! file_exists( $file ) ){
			return '';
		}
		list( $width, $height, $type, $attr ) = getimagesize( $file );
		if( $height < $width ){
			return $width;
		} else {
			return $height;
		}
	}
	
	function getImageInfo( $file ){
		if( ! file_exists( $file ) ){
			return '';
		}
		list( $width, $height ) = getimagesize( $file );
		return '{"width":' . $width . ', "height":' . $height 
				. ', "broadside":"' . ( $height < $width ? 'width' : 'height' ) . '"'
				. ', "broadlength":' . ( $height < $width ? $width : $height ) . '}';
	}
	
	function getInputHashSU() {
		if( !$_GET['-ses'] ){
			$query = html_entity_decode($_SERVER['QUERY_STRING']);
			parse_str( $query, $_GET );
		}
		$v = new virthos;
		$data = array(
			'user_id' => $v->vtglobal('pxtcSofubUserID'),
			'project_id' => $v->vtglobal('pxtcSofubProjectID'),
			'sender_holder' => '',
			'sender_account_number' => '',
			'sender_bank_code' => '',
			'sender_country_id' => '',
			'amount' => $v->vtget('Gesamtpreis|num-en:2.'),
			'currency_id' => 'EUR',
			'reason_1' => 'Bestell-Nr. ' . $v->vtsession('OrderNumber'),
			'reason_2' => '',
			'user_variable_0' => $_GET['-pg'],
			'user_variable_1' => $_GET['-ses'],
			'user_variable_2' => $_GET['-cod'],
			'user_variable_3' => $v->vtsession('OrderPageID'),
			'user_variable_4' => '',
			'user_variable_5' => '',
			'project_password' => $v->vtglobal('pxtcSofubPassword')
		);
		$data_implode = implode('|', $data);
		$hash = md5($data_implode);
		return $hash;
	}
	
	function getConfirmationHashSU() {
		$v = new virthos;
		if( $v->vtglobal('pxtcSofubNotificationPassword') != '' ){
			$password = $v->vtglobal('pxtcSofubNotificationPassword');
		} else {
			$password = $v->vtglobal('pxtcSofubPassword');
		}
		$data = array( 
		 'transaction' => $_POST['transaction'], 
		 'user_id' => $_POST['user_id'], 
		 'project_id' => $_POST['project_id'], 
		 'sender_holder' => $_POST['sender_holder'], 
		 'sender_account_number' => $_POST['sender_account_number'], 
		 'sender_bank_code' => $_POST['sender_bank_code'], 
		 'sender_bank_name' => $_POST['sender_bank_name'], 
		 'sender_bank_bic' => $_POST['sender_bank_bic'], 
		 'sender_iban' => $_POST['sender_iban'], 
		 'sender_country_id' => $_POST['sender_country_id'], 
		 'recipient_holder' => $_POST['recipient_holder'], 
		 'recipient_account_number' => $_POST['recipient_account_number'], 
		 'recipient_bank_code' => $_POST['recipient_bank_code'], 
		 'recipient_bank_name' => $_POST['recipient_bank_name'], 
		 'recipient_bank_bic' => $_POST['recipient_bank_bic'], 
		 'recipient_iban' => $_POST['recipient_iban'], 
		 'recipient_country_id' => $_POST['recipient_country_id'], 
		 'international_transaction' => $_POST['international_transaction'], 
		 'amount' => $_POST['amount'], 
		 'currency_id' => $_POST['currency_id'], 
		 'reason_1' => $_POST['reason_1'], 
		 'reason_2' => $_POST['reason_2'], 
		 'security_criteria' => $_POST['security_criteria'], 
		 'user_variable_0' => $_POST['user_variable_0'], 
		 'user_variable_1' => $_POST['user_variable_1'], 
		 'user_variable_2' => $_POST['user_variable_2'], 
		 'user_variable_3' => $_POST['user_variable_3'], 
		 'user_variable_4' => $_POST['user_variable_4'], 
		 'user_variable_5' => $_POST['user_variable_5'], 
		 'created' => $_POST['created'], 
		 'password' => $password 
		); 
		$data_implode = implode('|', $data); 
		$hash = md5($data_implode);
		return $hash;		
	}

	function verifyPaypalIPNMessage(){
		$req = 'cmd=_notify-validate'; 
		if( function_exists( 'get_magic_quotes_gpc' ) ){
			$get_magic_quotes_exists = true;
		} 
		foreach( $_POST as $key => $value ){
		// Handle escape characters, which depends on setting of magic quotes 
			if( $get_magic_quotes_exists == true && get_magic_quotes_gpc() == 1 ){
				$value = urlencode( stripslashes( $value ) ); 
			} else { 
				$value = urlencode( $value ); 
			} 
			$req .= "&$key=$value"; 
		}
		
		// Post back to PayPal to validate 
		$header .= "POST /cgi-bin/webscr HTTP/1.1\r\n";
		$header .= "Content-Type: application/x-www-form-urlencoded\r\n";
		$header .= "Host: www.paypal.com\r\n";
		$header .= "Content-Length: " . strlen( $req ) . "\r\n";
		$header .= "Connection: close\r\n\r\n";
		$fp = fsockopen( 'www.paypal.com', 80, $errno, $errstr, 30 );
 
		// Process validation from PayPal 
		if( !$fp ){ 
			return 'ERROR 1';
		} else { 
			fwrite( $fp, $header . $req );
			while( !feof($fp) ) { 
			  $res .= fgets( $fp, 128 );
			}
			fclose( $fp );

			$lines = explode( "\n", $res );
			end( $lines );
			return current( $lines );
		}
	}
	
	function createPassword( $length = 10 ){
		$characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.:,;#+*?=()/$!';
		if( $length > 0 && $length < 50 ){
			$pw = '';
			while( strlen( $pw ) < $length ){
				$pw .= $characters[ rand( 0, strlen( $characters ) - 1 ) ];
			}
			return $pw;
		}
		return false;
	}

	function analyzeSearchString(){
		if( empty( $_REQUEST['-search'] ) ){
			return '';
		}
		if( is_array( $_REQUEST['-search'] ) ){
			$_REQUEST['-search'] = implode( ' ', $_REQUEST['-search'] );
		}
		if( get_magic_quotes_gpc() ){
			$_REQUEST['-search'] = stripslashes( $_REQUEST['-search'] );
		}
		$matches = array();
		$result['all'] = '';
		preg_match_all( '/-?([a-zA-Z0-9_]+:)?(("[^"]+")|([\S]+))/', $_REQUEST['-search'], $matches );
		foreach( $matches[0] as $criteria ){
			if( preg_match( '/^(-?[a-zA-Z0-9_]+):/', $criteria, $field ) ){
				if( empty( $result[ strtolower($field[1]) ] ) ){
					$result[ strtolower($field[1]) ] = utf8_encode( substr( $criteria, strlen( $field[1] ) + 1 ) ) . ' ';				
				} else {
					$result[ strtolower($field[1]) ] .= utf8_encode( substr( $criteria, strlen( $field[1] ) + 1 ) ) . ' ';
				}
			} else {
				$result['all'] .= utf8_encode( $criteria ) . ' ';
			}
		}
		foreach( $result as $key => $value ){
			$result[ $key ] = str_replace( '*', '%', $value );
		}
		return $result;
	}
	
	function isZipAvailable() {
		$v = new virthos;
		$cmd = $v->vtconfig('pathToZipCommand') ? $v->vtconfig('pathToZipCommand') : '/usr/bin/zip';
		exec( $cmd . ' -L', $output );
		if( strpos( implode( $output ), 'Copyright' ) !== FALSE ){
			return 1;
		} else {
			return 0;
		}
	}
	
	function checkEnvironment() {
		$v = new virthos;
		$result = array();

		if( !function_exists( 'imagecreatetruecolor' ) ){
			$result[] = 'In der PHP-Umgebung steht keine GD-Library zur Verfgung. Sprechen Sie Ihren Provider oder Administrator an, damit diese nachinstalliert wird.';
		}
		$thumbnailsDir = $v->vtconfig('pathToData') . 'pixtacy_thumbnails';
		if( !file_exists( $thumbnailsDir ) ){
			mkdir( $thumbnailsDir );
			chmod( $thumbnailsDir, 0777 );
		}
		if( !file_exists( $thumbnailsDir ) ){
			$result[] = 'Thumbnails-Verzeichnis konnte nicht erstellt werden. Bitte erweitern Sie die Zugriffsrechte des \'data\'-Verzeichnisses mit Hilfe Ihres FTP-Programms (chmod 777).';
		} else {
			$tempDir = $thumbnailsDir . '/.temp';
			if( file_exists( $tempDir ) ){
				if( !rmdir( $tempDir ) ){
					$result[] = 'Temporres Verzeichnis data/pixtacy_thumbnails/.temp konnte nicht gelscht werden. Bitte entfernen Sie dieses mit Hilfe Ihres FTP-Programms.';
				} else {
					mkdir( $tempDir, 0777 );
					if( !file_exists( $tempDir ) ){
						$result[] = 'Erstellen eines Unterverzeichnisses ist fehlgeschlagen. Vermutlich ist auf dem Server der PHP-Safe-mode aktiviert. Sprechen Sie Ihren Provider oder Administrator an, damit dieser abgeschaltet wird.';
					} else {
						$testFile = $tempDir . '/test';
						if( ! $fp = fopen( $testFile, 'w' ) ){
							$result[] = 'Erzeugen einer Testdatei ist fehlgeschlagen. Vermutlich ist auf dem Server der PHP-Safe-mode aktiviert. Sprechen Sie Ihren Provider oder Administrator an, damit dieser abgeschaltet wird.';
						} else {
							fclose( $fp );
							unlink( $testFile );
						}
						rmdir( $tempDir );
					}
				}
			}
		}
		return implode( '|', $result );
	}

	function checkDirectory( $dir, $create = FALSE ){
		// prft, ob das Verzeichnis $dir existiert und erstellt es, falls $create gesetzt ist
		// $dir muss relativ zum lores-Verzeichnis angegeben sein

		$v = new virthos;
		
		if( $dir == '' ){
			return -1;
		} elseif( $dir == '/' ){
			return 1;
		} elseif( $dir[0] == '/' && file_exists( $v->vtglobal('pxtcPathLores') . $dir ) ){
			return 1;
		} elseif( ! $create ){
			return -1;
		}

		if( $v->ftpConnect() < 0 ){
			return -3; // keine Verbindung zum FTP-Server mglich
		}

		if( ! $loresPath = convertToFtpPath( $v->vtglobal('pxtcPathLores') ) ){
			return -4; // Angabe zum lores-Verzeichnis fehlerhaft
		}

		$v->vtdo( array( '-act' => 'ftp', '-cmd' => 'chdir', '-remote' => $loresPath ) );
		if( $v->vtresult() != 1 ) {
			return -5; // kein Wechsel zum lores-Verzeichnis mglich
		}		
		
		$dir = trim( $dir, '/' );

		$v->vtdo( array( '-act' => 'ftp', '-cmd' => 'chdir', '-remote' => $dir ) );
		if( $v->vtresult() != 1 ){
			if( $create ){
				$v->vtdo( array( '-act' => 'ftp', '-cmd' => 'mkdir', '-remote' => $dir ) );
				if( $v->vtresult() != 1 ){
					return -6; // Erstellen des Verzeichnisses $dir fehlgeschlagen
				}
			} else {
				return -7; // Verzeichnis $dir existiert nicht
			}
		}

		return 1;
	}
	
	function convertToFtpPath( $path ){
		// konvertiert $path zu einem FTP-Pfad
		$v = new virthos;
		if( $path[0] == '/' ){
			return FALSE; // absolute Pfade sind nicht erlaubt
		}
		return rtrim( $v->vtget( 'ftpBasePath' ), '/' ) . '/' . $path;
	}

	function getFtpPaths( $path, $is_dir = FALSE ){
		// liefert eine Liste mit absoluten FTP-Pfaden zu allen Bilddateien bzw. -ordnern,
		// die zu $path gehren (lores, hires, thumbnails)
		// $path ist eine Pfadangabe relativ zum lores-Verzeichnis

		$v = new virthos;
		$result = array();
		if( $thumbsPath = $v->vtget('thumbnailURL') ){
			$result[] = convertToFtpPath( $thumbsPath ) . $path;
		}
		if( $loresPath = $v->vtglobal('pxtcPathLores') ){
			$result[] = convertToFtpPath( $loresPath ) . $path;
		}
		if( $hiresPath = $v->vtglobal('pxtcPathHires') ){
			if( $is_dir ){
				$result[] = convertToFtpPath( $hiresPath ) . $path;
			} else {
				$list = getHiresFiles( $path );
				if( ! empty( $list ) ){
					$hiresPath = convertToFtpPath( $hiresPath ) . dirname( $path );
					foreach( $list as $item ){
						$result[] = cleanPath( $hiresPath . '/' . $item['directory'] . '/' . $item['name'] );
					}
				}
			}
		}
		return $result;
	}

	function ftpDelete( $path, $is_dir = FALSE ){
		// lscht das Bild bzw. den Bilderordner $path vom Server (lores, hires, thumbnails)
		// $path muss relativ zum lores-Verzeichnis angegeben sein
		
		if( $path == '' || $path == '/' || strpos( $path, '../' ) !== FALSE ){
			return FALSE;
		}

		$v = new virthos;
		if( $v->ftpConnect() < 0 ){
			return -1; // keine Verbindung zum FTP-Server mglich
		}

		$list = getFtpPaths( $path, $is_dir );
		if( empty( $list ) ){
			return FALSE;
		}

		// Objekte lschen
		$counter = 0;
		foreach( $list as $item ){
			$v->vtdo( array( '-act' => 'ftp', '-cmd' => 'delete', '-remote' => $item, '-recursive' => '1' ) );
			if( $v->vtresult() == 1 ){
				$counter++;
			}
		}
		return $counter > 0 ? 1 : 0;
	}
	
	function ftpDeleteThumbs( $path ){
		// lscht zu einem Bild bzw. Bilderordner die zugehrigen Thumbnails vom Server
		// $path muss relativ zum lores-Verzeichnis angegeben sein
		
		if( $path == '' || $path == '/' || strpos( $path, '../' ) !== FALSE ){
			return FALSE;
		}

		$v = new virthos;
		if( $v->ftpConnect() < 0 ){
			return -1; // keine Verbindung zum FTP-Server mglich
		}

		if( ! $thumbsPath = $v->vtget('thumbnailURL') ){
			return -2; // Thumbnail-Pfad nicht gesetzt
		}
		
		$v->vtdo( array( 
						'-act' => 'ftp', 
						'-cmd' => 'delete', 
						'-remote' => convertToFtpPath( $thumbsPath ) . $path, 
						'-recursive' => '1' 
						) 
					);

		return $v->vtresult();
	}
	
	function getHiresFiles( $image ){
		// liefert eine Liste mit allen Hires-Dateien, die zum Bild $image existieren
		$v = new virthos;
		$name = basename( $image );
		$path = $v->vtglobal('pxtcPathHires') . '/' . trim( dirname( $image ), '/' ) . '/';
		$folderlist = getDirectoryList( $path, TRUE );
		if( $folderlist == -1 ){
			return '';
		}
		array_unshift( $folderlist, '/' );
		$result = array();
		foreach( $folderlist as $folder ){
			$filelist = getFileList( $path . $folder, '^' . preg_quote( substr( $name, 0, strrpos( $name, '.' ) ) ) . '\..+', TRUE );
			if( $filelist != '-1' ){
				foreach( $filelist as $file ){
					$filepath = $path . $folder . '/' . $file;
					$result[] = array( 
						'name' => $file, 
						'directory' => $folder == '/' ? '' : $folder, 
						'size' => getFileSize( $filepath ), 
						'extension' => strtolower( pathinfo( $filepath, PATHINFO_EXTENSION ) ), 
						'resolution' => implode( ' x ', array_slice( getimagesize( $filepath ), 0, 2 ) )
					);
				}
			}
		}
		return empty( $result ) ? '' : $result;
	}
	
	function cleanPath( $path, $removeTrailingSlash = FALSE ){
		$path = preg_replace( '|/+|', '/', $path );
		if( $removeTrailingSlash && substr( $path, -1 ) == '/' ){
			$path = substr( $path, 0, -1 );
		}
		return $path;
	}

	function transferToPxprintNet( $userId, $items, $key, $contact, $delivery, $subject = '' ){
		$timestamp = time();
		$request = 'shop_id=' . $userId . '&items=' . urlencode( $items ) . '&timestamp=' . $timestamp;
		$request .= '&hash=' . md5( $userId . $items . $timestamp . $key );
		$request .= '&base_url=http://' . $_SERVER["HTTP_HOST"] . dirname( $_SERVER['SCRIPT_NAME']);
		$request .= '/templates/pixtacy/modules/download.php?c=';
		
		$contact = json_decode( $contact, TRUE );
		$request .= '&inv_lastname=' . urlencode( $contact['lastname'] );
		$request .= '&inv_firstname=' . urlencode( $contact['firstname'] );
		$request .= '&inv_organization=' . urlencode( $contact['organization'] );
		$request .= '&inv_address=' . urlencode( $contact['address'] );
		$request .= '&inv_city=' . urlencode( $contact['city'] );
		$request .= '&inv_zipcode=' . urlencode( $contact['zipcode'] );
		$request .= '&inv_country=' . urlencode( $contact['country'] );
		$request .= '&inv_state=' . urlencode( $contact['state'] );
		$request .= '&inv_phone=' . urlencode( $contact['phone'] );
		$request .= '&inv_email=' . urlencode( $contact['email'] );
		
		if( $delivery == 'SHOP' ){
			$request .= '&ship_to_enduser=0' ;
		} else {
			$request .= '&ship_to_enduser=1' ;
			if( ! empty( $delivery ) ){
				$delivery = json_decode( $delivery, TRUE );
				$request .= '&del_lastname=' . urlencode( $delivery['lastname'] );
				$request .= '&del_firstname=' . urlencode( $delivery['firstname'] );
				$request .= '&del_organization=' . urlencode( $delivery['organization'] );
				$request .= '&del_address=' . urlencode( $delivery['address'] );
				$request .= '&del_city=' . urlencode( $delivery['city'] );
				$request .= '&del_zipcode=' . urlencode( $delivery['zipcode'] );
				$request .= '&del_country=' . urlencode( $delivery['country'] );
				$request .= '&del_state=' . urlencode( $delivery['state'] );
			}
		}
		
		$request .= '&subject=' . urlencode( $subject );

		$header .= "POST /index.php/api/create HTTP/1.0\r\n"; 
    	$header .= "Host: api.pxprint.net\r\n";
		$header .= "Content-Type: application/x-www-form-urlencoded\r\n"; 
		$header .= "Content-Length: " . strlen( $request ) . "\r\n"; 
		$header .= "Connection: close\r\n\r\n"; 

		$fp = fsockopen( 'api.pxprint.net', 80, $errno, $errstr, 30 );
		if( !$fp ){ 
			return '-1';
		} else { 
			fwrite( $fp, $header . $request );
			while( !feof( $fp ) ) { 
			  $result .= fgets( $fp, 128 );
			}
			fclose( $fp );
			$lines = explode( "\n", $result );
			end( $lines );
			if( strpos( $lines[0], '200' ) > 0 ){
				return current( $lines );
			} else {
				return '-2 ' . current( $lines );
			}
		}
	}

	function chmodr( $path, $filemode ){ 
		if( ! is_dir( $path ) ){
			return @chmod( $path, $filemode ); 
		}
		$dh = opendir( $path ); 
		while( ( $file = readdir( $dh ) ) !== false ){ 
			if( $file != '.' && $file != '..' ){
				$fullpath = $path . '/' . $file; 
				if( is_link( $fullpath ) ){
					return FALSE; 
				} elseif( !is_dir( $fullpath ) && ! @chmod( $fullpath, $filemode ) ){
					return FALSE; 
				} elseif( ! chmodr( $fullpath, $filemode ) ){
					return FALSE; 
				}
			} 
		} 
		closedir( $dh ); 
		if( @chmod( $path, $filemode ) ){
			return TRUE; 
		} else {
			return FALSE; 
		}
	}
	
	function createEasybillInvoice( $customer_id,  $customer_number )
	{
		if ( ! checkSecureTransferMethod() )
		{
			return -1; // der Server erlaubt keine sicheren Verbindungen zu anderen Diensten
		}

		$v = new virthos;
		$easybill = new Easybill( $v->vtglobal('pxtcEasybillApiKey'), $v->vtglobal('pxtcSecureTransferMethod') );
		$customer = array();

		### ggf. vorhandenen Kundendatensatz holen ###

		do
		{
			if ( $customer_id > 0 )
			{
				$response = $easybill->get_customer( $customer_id );
				if ( $response != FALSE )
				{
					$customer = $response;
					break;
				}
			}

			if ( $customer_number > 0 )
			{
				$response = $easybill->get_customer_by_number( $customer_number );
				if ( $response != FALSE )
				{
					$customer = $response;
					break;
				}
			}

			$response = $easybill->search_customers( $v->vtpage( 'Name' ) );
			if ( $response == FALSE )
			{
				break;
			}

			foreach ( $response as $record )
			{
				if ( strtolower( $record['firstName'] ) == strtolower( $v->vtpage( 'Vorname' ) )
						&& strtolower( $record['lastName'] ) == strtolower( $v->vtpage( 'Name' ) )
						&& strtolower( $record['companyName'] ) == strtolower( $v->vtpage( 'Firma' ) )
						&& strtolower( $record['street'] ) == strtolower( $v->vtpage( 'Strasse' ) )
						&& strtolower( $record['zipCode'] ) == strtolower( $v->vtpage( 'PLZ' ) )
						&& strtolower( $record['city'] ) == strtolower( $v->vtpage( 'Ort' ) )
						&& strtolower( $record['country'] ) == strtolower( $v->vtpage( 'Land' ) )
						&& strtolower( $record['email'] ) == strtolower( $v->vtpage( 'Email' ) ) )
				{
					$customer = $record;
					break 2;
				}
			}
		} 
		while ( FALSE );

		### Kundendatensatz erstellen bzw. aktualisieren ###

		$customer['salutation'] = utf8_encode( isset( $customer['salutation'] ) ? $customer['salutation'] : 0 );
		$customer['firstName'] = utf8_encode( $v->vtpage( 'Vorname' ) );
		$customer['lastName'] = utf8_encode( $v->vtpage( 'Name' ) );
		$customer['companyName'] = utf8_encode( $v->vtpage( 'Firma' ) );
		$customer['street'] = utf8_encode( $v->vtpage( 'Strasse' ) );
		$customer['zipCode'] = utf8_encode( $v->vtpage( 'PLZ' ) );
		$customer['city'] = utf8_encode( $v->vtpage( 'Ort' ) );
		$customer['country'] = utf8_encode( $v->vtpage( 'Land' ) );
		$customer['ustid'] = utf8_encode( $v->vtpage( 'USt_ID' ) );
		$customer['email'] = utf8_encode( $v->vtpage( 'Email' ) );

		$response = $easybill->set_customer( $customer );
		
		if ( empty( $response['customerID'] ) )
		{
			return array( 'error' => 'Kundendatensatz konnte nicht erstellt oder aktualisiert werden' );
		}

		$customer = $response;

		### Rechnungspositionen zusammenstellen ###

		$items = json_decode( $v->vtpage( 'Warenkorb' ), TRUE );
		$products = array();
		$max_vat_rate = 0;
		foreach ( $items as $item )
		{
			if ( isset( $products[ $item['product'] ] ) )
			{
				$products[ $item['product'] ]['count'] += $item['quantity'];
			}
			else
			{
				$products[ $item['product'] ] = array
					( 
						'positionType' => 'POSITION', 
						'itemNumber' => $item['product'], 
						'itemDescription' => utf8_encode( $item['title'] ), 
						'count' => $item['quantity'], 
						'unit' => '', 
						'singlePriceNetto' => 100 * $item['netprice'], 
						'ustPercent' => $item['vatrate']
					);
			}
			$max_vat_rate = $item['vatrate'] > $max_vat_rate ? $item['vatrate'] : $max_vat_rate;
		}

		### Versandkosten hinzufgen ###

		if ( $v->vtpage( 'Versandkosten_netto' ) > 0 )
		{
			$products[] = array
				( 
					'positionType' => 'POSITION', 
					'itemDescription' => utf8_encode( $v->vtget( 's_fld_shipping_costs' ) . ' (' . $v->vtpage( 'Liefermethode' ) . ')' ), 
					'count' => 1, 
					'unit' => '', 
					'singlePriceNetto' => 100 * $v->vtpage( 'Versandkosten_netto' ), 
					'ustPercent' => $max_vat_rate
				);
		}

		### Rechnungsdokument erstellen ###

		$document['customerID'] = $customer['customerID'];
		$document['type'] = 'INVOICE';
		$document['currency'] = $v->vtglobal('pxtcEasybillCurrency');
		$document['contactLabel'] = utf8_encode( $v->vtget('s_fld_order_number') );
		$document['contactText'] = utf8_encode( $v->vtpage('Nummer') );
		$document['documentPosition'] = array_values( $products );
		$document['templateName'] = utf8_encode( $v->vtglobal('pxtcEasybillTemplate_' . $v->vtpage('Language') ) );
		$document['text'] = utf8_encode( $v->vtget( 's_invoice_text_' . strtolower( $v->vtpage('Bezahlmethode') ) ) );
		$document['sendasemail'] = ( $v->vtglobal( 'pxtcEasybillAutoSend' ) == '1' );

		if ( $v->vtpage('nicht_steuerbar') == '1' )
		{
			$document['taxOptions'] = $v->vtpage('USt_ID') == '' ? 'nStb' : 'nStbUstID';
		}

		$response = $easybill->create_document( $document );

		if ( empty( $response['documentID'] ) )
		{
			return array( 'error' => 'Rechnung konnte nicht erstellt werden' );
		}

		$result = array( 
			'document_id' => $response['documentID'], 
			'document_number' => $response['documentNumber'],
			'customer_id' => $customer['customerID'],
			'customer_number' => $customer['customerNumber']
			);

		return $result;

	}


	function checkSecureTransferMethod( $force = FALSE )
	{
		$v = new virthos;
		
		if ( in_array( $v->vtglobal('pxtcSecureTransferMethod'), array( 'socket', 'curl' ) ) 
				&& ! $force )
		{
			return $v->vtglobal('pxtcSecureTransferMethod');
		}

		if ( function_exists( 'fsockopen' ) )
		{
			$fp = @fsockopen( "ssl://www.google.com", 443 );
			if ( $fp )
			{
				fclose( $fp );
				$v->vtdo( array( '-act' => 'update', '-obj' => 70, 'pxtcSecureTransferMethod' => 'socket' ) );
				return 'socket';
			}
		}

		if ( function_exists( 'curl_init' ) )
		{
			$curl = curl_init();
			curl_setopt( $curl, CURLOPT_URL, "https://www.google.com/" );
			curl_setopt( $curl, CURLOPT_HEADER, 0 );
			curl_setopt( $curl, CURLOPT_RETURNTRANSFER, 1 );
			$result = curl_exec( $curl );
			curl_close($curl);
			if ( $result !== FALSE )
			{
				$v->vtdo( array( '-act' => 'update', '-obj' => 70, 'pxtcSecureTransferMethod' => 'curl' ) );
				return 'curl';
			}
		}

		return FALSE;
	}

	function check_vat_number( $number, $country = '' )
	{
		$codes = 'AT|BE|BG|CY|CZ|DE|DK|EE|EL|ES|FI|FR|GB|HU|IE|IT|LT|LU|LV|MT|NL|PL|PT|RO|SE|SI|SK';

		$number = str_replace( ' ', '', $number );

		if ( $number == '' || strlen( $number ) < 10 || strlen( $number ) > 15 )
		{
			return FALSE;
		}

		if ( $country == '' )
		{
			if ( ! in_array( substr( $number, 0, 2 ), explode( '|', $codes ) ) )
			{
				return FALSE;
			}
		}
		else
		{
			if ( $country == 'GR' )
			{
				$code = 'EL';
			}
			else
			{
				$code = strtoupper( $country );
			}

			if ( substr( $number, 0, 2 ) != $code )
			{
				return FALSE;
			}
		}

		return TRUE;
	}

	function addCleverreachReceiver( $email )
	{
		if ( ! checkSecureTransferMethod() )
		{
			return -1; // der Server erlaubt keine sicheren Verbindungen zu anderen Diensten
		}

		require_once dirname( __FILE__ ) . '/cleverreach.class.php';
		$v = new virthos;
		$cleverreach = new Cleverreach( $v->vtglobal('pxtcCleverreachId'), $v->vtglobal('pxtcCleverreachApiKey'), $v->vtglobal('pxtcSecureTransferMethod') );
		return $cleverreach->addReceiver( $v->vtglobal('pxtcCleverreachListId'), $email );
	}

	function myhtmlentities( $str )
	{
		return htmlentities( $str, ENT_COMPAT | ENT_HTML401, 'ISO-8859-1' );
	}
?>